<?php

namespace PAR\Service;

use DASPRiD\EnumTest\NullValueTest;
use PAR\Factory\BusinessPartnerFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use PAR\Service\FileServerService;

class BusinessPartnerService
{

    protected $connection;
    protected $businessPartnerFactory;
    protected $environment;
    protected $newCode;
    protected $FileServerService;
    protected $typeOfPartner = array(
        "CPF" => "F",
        "CNPJ" => "J",
        "LIVRE" => "I",
        "CODIGO" => "O"
    );

    public function __construct(BusinessPartnerFactory $businessPartnerFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager, FileServerService $FileServerService)
    {
        $this->businessPartnerFactory = $businessPartnerFactory;
        $this->connection             = $entityManager->getConnection();
        $this->environment            = $environment;
        $this->newCode                = $newCode;
        $this->FileServerService      = $FileServerService;
    }

    public function getBusinessPartner()
    {
        return $this->businessPartnerFactory->getBusinessPartner();
    }

    public function getOperator()
    {
        return $this->businessPartnerFactory->getOperator();
    }

    public function getPartnerClassification()
    {
        return $this->businessPartnerFactory->getPartnerClassification();
    }

    public function getTypeOfPartner()
    {
        return $this->businessPartnerFactory->getTypeOfPartner();
    }

    public function getMarketSegment()
    {
        return $this->businessPartnerFactory->getMarketSegment();
    }

    public function getBranchOfActivity()
    {
        return $this->businessPartnerFactory->getBranchOfActivity();
    }

    public function getTypeOfActivity()
    {
        return $this->businessPartnerFactory->getTypeOfActivity();
    }

    public function getSpecificBusinessPartner($parametros)
    {
        return $this->businessPartnerFactory->getSpecificBusinessPartner($parametros);
    }

    public function validarInscricaoDuplicada($nrinscricaoparc, array $cdtipoparceiro, $nrparcnegocio = null)
    {
        $this->validarInscricaoDuplicadaParceiro($nrinscricaoparc, $nrparcnegocio);
        if (in_array("CLIENTE", $cdtipoparceiro)) {
            $this->validarInscricaoDuplicadaCliente($nrinscricaoparc);
        }
    }

    private function validarInscricaoDuplicadaParceiro($nrinscricaoparc, $nrparcnegocio)
    {
        $params = array(
            "NRINSCRICAOPARC" => $nrinscricaoparc,
            "NRPARCNEGOCIO"   => $nrparcnegocio
        );
        $dataset = $this->businessPartnerFactory->getParceiroPorInscricao($params);
        if (count($dataset) > 0) {
            throw new \Exception("O número de inscrição informado já pertence ao seguinte parceiro de negócio: <br/>" . $dataset[0]["NMPRINCIPALPARC"]);
        }
    }

    private function validarInscricaoDuplicadaCliente($nrinscricaoparc)
    {
        $params = array(
            "NRINSJURCLIE" => $nrinscricaoparc
        );
        $dataset = $this->businessPartnerFactory->getClienteSemParceiroPorInscricao($params);
        if (count($dataset) > 0) {
            throw new \Exception("O número de inscrição informado já pertence a um cliente em que não há um parceiro de negócio relacionado. Cliente: <br/>" . $dataset[0]["NMRAZSOCCLIE"]);
        }
    }

    public function insertNewBusinessPartner($parametros)
    {
        $this->validarInscricaoDuplicada($parametros['NRINSCRICAOPARC'], $parametros['CDTIPOPARCEIRO']);

        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();
        try {
            // insere parceiro
            $nrparcnegocio = $this->newCode->getCodeByTableName("PARCNEGOCIO", $nrorg, 1, 10, true);
            $dataParam = array(
                "NRPARCNEGOCIO"      => $nrparcnegocio,
                "NRORG"              => $nrorg,
                "CDSEGMER"           => $parametros['CDSEGMER'],
                "NMPRINCIPALPARC"    => $parametros['NMPRINCIPALPARC'],
                "NMSECUNDARIPARC"    => $parametros['NMSECUNDARIPARC'],
                "DTNASCIFUNDPARC"    => ($parametros['DTNASCIFUNDPARC'] == '') ? null : $parametros['DTNASCIFUNDPARC'],
                "NRINSCRICAOPARC"    => $parametros['NRINSCRICAOPARC'],
                "NRORGINCLUSAO"      => $nrorg,
                "CDOPERINCLUSAO"     => $cdoperador,
                "CDTIPOPARCPRINCIPAL" => $parametros['CDTIPOPARCPRINCIPAL'],
                "CDTIPOINSCRICAO"    => $parametros['CDTIPOINSCRICAO'],
                "IDPESSOAFISICA"     => $parametros['IDPESSOAFISICA'],
                "IDINSTITUICAO"      => $parametros['IDINSTITUICAO'],
                "CDATIVCLI"          => $parametros['CDATIVCLI'],
                "NRSEQTPATIVCLI"     => $parametros['NRSEQTPATIVCLI'],
                "CDOPERRESPONSAVEL"  => $parametros['CDOPERRESPONSAVEL'],
                "NRTPCLASSIFICAPARC" => $parametros['NRTPCLASSIFICAPARC'],
                "CDINSCESTA"         => $parametros['CDINSCESTA'],
                "CDINSCMUNI"         => $parametros['CDINSCMUNI'],
                "CDIDIOM"            => $parametros['CDIDIOM'],
                "NRMOEDA"            => $parametros['NRMOEDA']
            );

            $this->businessPartnerFactory->insertNewBusinessPartner($dataParam);

            // insere endereco
            $nrenderecoparc = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);
            $dataAddress = array(
                "NRENDERECOPARC"   => $nrenderecoparc,
                "NRPARCNEGOCIO"    => $nrparcnegocio,
                "CDTIPOENDERECO"   => $parametros['CDTIPOENDERECO'],
                "NRORG"            => $nrorg,
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador,
                "CDPAIS"           => $parametros['CDPAIS'],
                "SGESTADO"         => $parametros['SGESTADO'],
                "CDMUNICIPIO"      => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"         => $parametros['CDBAIRRO'],
                "NMBAIRROENDERECO" => $parametros['NMBAIRROENDERECO'],
                "DSCOMPLEENDERECO" => $parametros['DSCOMPLEENDERECO'],
                "NRCEPENDERECO"    => $parametros['NRCEPENDERECO'],
                "DSENDERECO"       => $parametros['DSENDERECO'],
                "NRIMOVELENDERECO" => $parametros['NRIMOVELENDERECO'],
                "DSREFERENCIAENDE" => $parametros['DSREFERENCIAENDE']
            );

            $this->businessPartnerFactory->insertNewAddress($dataAddress);

            // insere parceria
            /*foreach($parametros['CDTIPOPARCEIRO'] as $typePartner) {
                $this->insertPARCERIA($nrparcnegocio, $typePartner, $nrorg, $cdoperador);
            }*/

            // replica registro para os tipos de parceiro
            $this->replicaParceiro($parametros, $nrparcnegocio, $nrorg, $cdoperador);
            $this->replicaEndereco($parametros, $nrenderecoparc, $nrparcnegocio, $nrorg, $cdoperador);

            $this->connection->commit();
            $response = 'OK';
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
            //$response = $e->getMessage();
        }

        return $response;
    }

    private function replicaParceiro($parametros, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio
        );
        if (in_array("CLIENTE", $parametros['CDTIPOPARCEIRO'])) {
            $dataset = $this->businessPartnerFactory->getQtdClienteParceiro($params);
            if ((int)$dataset[0]["QTD"] > 0) {
                $this->updateCLIENTE($parametros, $nrorg, $cdoperador);
            } else {
                $this->insertCLIENTE($parametros, $nrparcnegocio, $nrorg, $cdoperador);
            }
        } else {
            $this->inativarCliente($nrparcnegocio, $nrorg, $cdoperador);
        }

        if (in_array("FORNECEDOR", $parametros['CDTIPOPARCEIRO'])) {
            $dataset = $this->businessPartnerFactory->getQtdFornecedorParceiro($params);
            if ((int)$dataset[0]["QTD"] > 0) {
                $this->updateFORNECEDOR($parametros, $nrorg, $cdoperador);
            } else {
                $this->insertFORNECEDOR($parametros, $nrparcnegocio, $nrorg, $cdoperador);
            }
        } else {
            $this->inativarFornecedor($nrparcnegocio, $nrorg, $cdoperador);
        }

        if (in_array("VENDEDOR", $parametros['CDTIPOPARCEIRO'])) {
            $dataset = $this->businessPartnerFactory->getQtdVendedorParceiro($params);
            if ((int)$dataset[0]["QTD"] > 0) {
                $this->updateVENDEDOR($parametros, $nrorg, $cdoperador);
            } else {
                $this->insertVENDEDOR($parametros, $nrparcnegocio, $nrorg, $cdoperador);
            }
        } else {
            $this->inativarVendedor($nrparcnegocio, $nrorg, $cdoperador);
        }

        if (in_array("PROSPECT", $parametros['CDTIPOPARCEIRO'])) {
            $dataset = $this->businessPartnerFactory->getQtdProspectParceiro($params);
            if ((int)$dataset[0]["QTD"] > 0) {
                $this->updatePROSPECT($parametros, $nrorg, $cdoperador);
            } else {
                $this->insertPROSPECT($parametros, $nrparcnegocio, $nrorg, $cdoperador);
            }
        } else {
            $this->inativarProspect($nrparcnegocio, $nrorg, $cdoperador);
        }

        foreach ($parametros['CDTIPOPARCEIRO'] as $type) {
            $data = array(
                "NRPARCNEGOCIO"  => $nrparcnegocio,
                "CDTIPOPARCEIRO" => $type
            );

            $dataset = $this->businessPartnerFactory->getQtdParceria($data);
            if ((int)$dataset[0]["QTD"] == 0) {
                $this->insertPARCERIA($nrparcnegocio, $type, $nrorg, $cdoperador);
            } else {
                $this->updateParceria("S", $nrparcnegocio, $type, $nrorg, $cdoperador);
            }
        }

        $this->updateParceriaNaoSelecionados($parametros['CDTIPOPARCEIRO'], $nrparcnegocio, $nrorg, $cdoperador);
    }

    private function replicaEndereco($parametros, $nrenderecoparc, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $conversaoTipoEndereco = array(
            'PRINCIPAL'  => 'P',
            'COBRANCA'   => 'C',
            'COMERCIAL'  => 'M',
            'ENTREGA'    => 'T',
            'EXTERIOR'   => 'E',
            'EXPORTACAO' => 'X',
            'OUTROS'     => 'O'
        );
        $tipoEndereco = $conversaoTipoEndereco[$parametros["CDTIPOENDERECO"]];
        // $tipoEnderecoOld = $conversaoTipoEndereco[$parametros["CDTIPOENDERECO_OLD"]];
        $tipoEnderecoOld = isset($parametros["CDTIPOENDERECO_OLD"]) ? $conversaoTipoEndereco[$parametros["CDTIPOENDERECO_OLD"]] : $tipoEndereco;

        $paramsaux = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
        );

        if (in_array("CLIENTE", $parametros['CDTIPOPARCEIRO'])) {
            $dataset = $this->businessPartnerFactory->getCodigoClienteParceiro($paramsaux);
            if (count($dataset) > 0) {
                $cdclienteparceiro = $dataset[0]["CDCLIENTE"];
                $params = array(
                    "CDCLIENTE"    => $cdclienteparceiro,
                    "IDTPENDECLIE" => $tipoEnderecoOld,
                );
                $dataset = $this->businessPartnerFactory->getQtdEnderecoClienteParceiro($params);
                if ((int)$dataset[0]["QTD"] > 0) {
                    $this->updateENDECLIE($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdclienteparceiro);
                } else {
                    $this->insertENDECLIE($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador);
                }
            }
        }

        $tiposEnderecoFornecedor = array('P', 'C', 'E', 'X');
        if (in_array("FORNECEDOR", $parametros['CDTIPOPARCEIRO']) && in_array($tipoEndereco, $tiposEnderecoFornecedor) && in_array($tipoEnderecoOld, $tiposEnderecoFornecedor)) {
            $dataset = $this->businessPartnerFactory->getCodigoFornecedorParceiro($paramsaux);
            if (count($dataset) > 0) {
                $cdfornecedorparceiro = $dataset[0]["CDFORNECED"];
                $params = array(
                    "CDFORNECED"   => $cdfornecedorparceiro,
                    "IDTPENDEFORN" => $tipoEnderecoOld,
                );
                $dataset = $this->businessPartnerFactory->getQtdEnderecoFornecedorParceiro($params);
                if ((int)$dataset[0]["QTD"] > 0) {
                    $this->updateENDEFORN($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdfornecedorparceiro);
                } else {
                    $this->insertENDEFORN($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador);
                }
            }
        }

        $tiposEnderecoVendedor = array('P');
        if (in_array("VENDEDOR", $parametros['CDTIPOPARCEIRO']) && in_array($tipoEndereco, $tiposEnderecoVendedor) && in_array($tipoEnderecoOld, $tiposEnderecoVendedor)) {
            $dataset = $this->businessPartnerFactory->getCodigoVendedorParceiro($paramsaux);
            if (count($dataset) > 0) {
                $cdvendedorparceiro = $dataset[0]["CDVENDEDOR"];
                $params = array(
                    "CDVENDEDOR"  => $cdvendedorparceiro,
                    "IDTPENDEVEN" => $tipoEnderecoOld,
                );
                $dataset = $this->businessPartnerFactory->getQtdEnderecoVendedorParceiro($params);
                if ((int)$dataset[0]["QTD"] > 0) {
                    $this->updateENDEVENDE($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdvendedorparceiro);
                } else {
                    $this->insertENDEVENDE($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador);
                }
            }
        }

        $tiposEnderecoProspect = array('P', 'C', 'E', 'X', 'T', 'O');
        if (in_array("PROSPECT", $parametros['CDTIPOPARCEIRO']) && in_array($tipoEndereco, $tiposEnderecoProspect) && in_array($tipoEnderecoOld, $tiposEnderecoProspect)) {
            $dataset = $this->businessPartnerFactory->getCodigoProspectParceiro($paramsaux);
            if (count($dataset) > 0) {
                $cdprospectparceiro = $dataset[0]["CDPROSPECT"];
                $params = array(
                    "CDPROSPECT"   => $cdprospectparceiro,
                    "IDTPENDEPROS" => $tipoEnderecoOld,
                );
                $dataset = $this->businessPartnerFactory->getQtdEnderecoProspectParceiro($params);
                if ((int)$dataset[0]["QTD"] > 0) {
                    $this->updateENDEPROSPECT($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdprospectparceiro);
                } else {
                    $this->insertENDEPROSPECT($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador);
                }
            }
        }
    }

    public function insertENDECLIE($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador)
    {
        $cdcliente = $this->getCodigoClienteParceiro($nrparcnegocio);
        if (!empty($cdcliente)) {
            $dsendereco = $parametros["DSENDERECO"];
            if (!empty($parametros["NRIMOVELENDERECO"])) {
                $dsendereco .= ", " . $parametros["NRIMOVELENDERECO"];
            }

            $params = array(
                "NRENDERECOPARC" => $nrenderecoparc,
                "CDCLIENTE"      => $cdcliente,
                "IDTPENDECLIE"   => $tipoEndereco,
                "CDPAIS"         => $parametros["CDPAIS"],
                "SGESTADO"       => $parametros["SGESTADO"],
                "CDMUNICIPIO"    => $parametros["CDMUNICIPIO"],
                "DSENDECLIE"     => $dsendereco,
                "CDBAIRRO"       => $parametros["CDBAIRRO"],
                "DSCOMPENDCLI"   => $parametros["DSCOMPLEENDERECO"],
                "NMBAIRCLIE"     => $parametros["NMBAIRROENDERECO"],
                "NRCEPCLIE"      => $parametros["NRCEPENDERECO"],
                "IDATIVO"        => "S",
                "NRORG"          => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador,
                "NRORGINCLUSAO"  => $nrorg
            );
            $this->businessPartnerFactory->insertENDECLIE($params);
        }
    }

    public function updateENDECLIE($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdcliente)
    {
        $dsendereco = $parametros["DSENDERECO"];
        if (!empty($parametros["NRIMOVELENDERECO"])) {
            $dsendereco .= ", " . $parametros["NRIMOVELENDERECO"];
        }

        $params = array(
            "IDTPENDECLIE"   => $tipoEndereco,
            "CDPAIS"         => $parametros["CDPAIS"],
            "SGESTADO"       => $parametros["SGESTADO"],
            "CDMUNICIPIO"    => $parametros["CDMUNICIPIO"],
            "DSENDECLIE"     => $dsendereco,
            "CDBAIRRO"       => $parametros["CDBAIRRO"],
            "DSCOMPENDCLI"   => $parametros["DSCOMPLEENDERECO"],
            "NMBAIRCLIE"     => $parametros["NMBAIRROENDERECO"],
            "NRCEPCLIE"      => $parametros["NRCEPENDERECO"],
            "IDATIVO"        => "S",
            "CDOPERULTATU"   => $cdoperador,
            "NRORGULTATU"    => $nrorg,
            "NRENDERECOPARC" => $nrenderecoparc,
            "CDCLIENTE"      => $cdcliente,
            "IDTPENDECLIE_OLD" => $tipoEnderecoOld
        );

        $this->businessPartnerFactory->updateENDECLIE($params);
    }

    public function insertENDEFORN($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador)
    {
        $cdforneced = $this->getCodigoFornecedorParceiro($nrparcnegocio);
        if (!empty($cdforneced)) {
            $dsendereco = $parametros["DSENDERECO"];
            if (!empty($parametros["NRIMOVELENDERECO"])) {
                $dsendereco .= ", " . $parametros["NRIMOVELENDERECO"];
            }

            $params = array(
                "NRENDERECOPARC" => $nrenderecoparc,
                "CDFORNECED"     => $cdforneced,
                "IDTPENDEFORN"   => $tipoEndereco,
                "CDPAIS"         => $parametros["CDPAIS"],
                "SGESTADO"       => $parametros["SGESTADO"],
                "CDMUNICIPIO"    => $parametros["CDMUNICIPIO"],
                "DSENDEFORN"     => $dsendereco,
                "DSCOMPENDFOR"   => $parametros["DSCOMPLEENDERECO"],
                "NMBAIRFORN"     => $parametros["NMBAIRROENDERECO"],
                "NRCEPFORN"      => $parametros["NRCEPENDERECO"],
                "IDATIVO"        => "S",
                "NRORG"          => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador,
                "NRORGINCLUSAO"  => $nrorg
            );
            $this->businessPartnerFactory->insertENDEFORN($params);
        }
    }

    public function updateENDEFORN($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdfornecedor)
    {
        $dsendereco = $parametros["DSENDERECO"];
        if (!empty($parametros["NRIMOVELENDERECO"])) {
            $dsendereco .= ", " . $parametros["NRIMOVELENDERECO"];
        }

        $params = array(
            "IDTPENDEFORN"   => $tipoEndereco,
            "CDPAIS"         => $parametros["CDPAIS"],
            "SGESTADO"       => $parametros["SGESTADO"],
            "CDMUNICIPIO"    => $parametros["CDMUNICIPIO"],
            "DSENDEFORN"     => $dsendereco,
            "DSCOMPENDFOR"   => $parametros["DSCOMPLEENDERECO"],
            "NMBAIRFORN"     => $parametros["NMBAIRROENDERECO"],
            "NRCEPFORN"      => $parametros["NRCEPENDERECO"],
            "IDATIVO"        => $parametros["IDATIVO"],
            "CDOPERULTATU"   => $cdoperador,
            "NRORGULTATU"    => $nrorg,
            "NRENDERECOPARC" => $nrenderecoparc,
            "CDFORNECED"      => $cdfornecedor,
            "IDTPENDEFORN_OLD" => $tipoEnderecoOld
        );

        $this->businessPartnerFactory->updateENDEFORN($params);
    }

    public function insertENDEVENDE($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador)
    {
        $cdvendedor = $this->getCodigoVendedorParceiro($nrparcnegocio);
        if (!empty($cdvendedor)) {
            $dsendereco = $parametros["DSENDERECO"];
            if (!empty($parametros["NRIMOVELENDERECO"])) {
                $dsendereco .= ", " . $parametros["NRIMOVELENDERECO"];
            }

            if (!empty($parametros["DSCOMPLEENDERECO"])) {
                $dsendereco .= " - " . $parametros["DSCOMPLEENDERECO"];
            }

            $params = array(
                "NRENDERECOPARC" => $nrenderecoparc,
                "CDVENDEDOR"     => $cdvendedor,
                "IDTPENDEVEN"    => $tipoEndereco,
                "CDPAIS"         => $parametros["CDPAIS"],
                "SGESTADO"       => $parametros["SGESTADO"],
                "CDMUNICIPIO"    => $parametros["CDMUNICIPIO"],
                "DSENDEVEN"      => $dsendereco,
                "CDBAIRRO"       => $parametros["CDBAIRRO"],
                "NMBAIRVEN"      => $parametros["NMBAIRROENDERECO"],
                "NRCEPVEN"       => $parametros["NRCEPENDERECO"],
                "IDATIVO"        => "S",
                "NRORG"          => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador,
                "NRORGINCLUSAO"  => $nrorg
            );
            $this->businessPartnerFactory->insertENDEVENDE($params);
        }
    }

    public function updateENDEVENDE($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdvendedor)
    {
        $dsendereco = $parametros["DSENDERECO"];
        if (!empty($parametros["NRIMOVELENDERECO"])) {
            $dsendereco .= ", " . $parametros["NRIMOVELENDERECO"];
        }

        if (!empty($parametros["DSCOMPLEENDERECO"])) {
            $dsendereco .= " - " . $parametros["DSCOMPLEENDERECO"];
        }

        $params = array(
            "IDTPENDEVEN"    => $tipoEndereco,
            "CDPAIS"         => $parametros["CDPAIS"],
            "SGESTADO"       => $parametros["SGESTADO"],
            "CDMUNICIPIO"    => $parametros["CDMUNICIPIO"],
            "DSENDEVEN"      => $dsendereco,
            "CDBAIRRO"       => $parametros["CDBAIRRO"],
            "NMBAIRVEN"      => $parametros["NMBAIRROENDERECO"],
            "NRCEPVEN"       => $parametros["NRCEPENDERECO"],
            "IDATIVO"        => $parametros["IDATIVO"],
            "CDOPERULTATU"   => $cdoperador,
            "NRORGULTATU"    => $nrorg,
            "NRENDERECOPARC" => $nrenderecoparc,
            "CDVENDEDOR"      => $cdvendedor,
            "IDTPENDEVEN_OLD" => $tipoEnderecoOld
        );

        $this->businessPartnerFactory->updateENDEVENDE($params);
    }

    public function insertENDEPROSPECT($parametros, $tipoEndereco, $nrparcnegocio, $nrenderecoparc, $nrorg, $cdoperador)
    {
        $cdprospect = $this->getCodigoProspectParceiro($nrparcnegocio);
        if (!empty($cdprospect)) {
            $params = array(
                "NRENDERECOPARC"   => $nrenderecoparc,
                "CDPROSPECT"       => $cdprospect,
                "IDTPENDEPROS"     => $tipoEndereco,
                "CDPAIS"           => $parametros["CDPAIS"],
                "SGESTADO"         => $parametros["SGESTADO"],
                "CDMUNICIPIO"      => $parametros["CDMUNICIPIO"],
                "DSENDEPROS"       => $parametros["DSENDERECO"],
                "CDBAIRRO"         => $parametros["CDBAIRRO"],
                "NMBAIRPROS"       => $parametros["NMBAIRROENDERECO"],
                "NRCEPPROS"        => $parametros["NRCEPENDERECO"],
                "NRIMOVELENDERECO" => $parametros["NRIMOVELENDERECO"],
                "IDATIVO"          => "S",
                "NRORG"            => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador,
                "NRORGINCLUSAO"    => $nrorg
            );
            $this->businessPartnerFactory->insertENDEPROSPECT($params);
        }
    }

    public function updateENDEPROSPECT($parametros, $tipoEndereco, $tipoEnderecoOld, $nrenderecoparc, $nrorg, $cdoperador, $cdprospect)
    {
        $params = array(
            "IDTPENDEPROS"     => $tipoEndereco,
            "CDPAIS"           => $parametros["CDPAIS"],
            "SGESTADO"         => $parametros["SGESTADO"],
            "CDMUNICIPIO"      => $parametros["CDMUNICIPIO"],
            "DSENDEPROS"       => $parametros["DSENDERECO"],
            "CDBAIRRO"         => $parametros["CDBAIRRO"],
            "NMBAIRPROS"       => $parametros["NMBAIRROENDERECO"],
            "NRCEPPROS"        => $parametros["NRCEPENDERECO"],
            "NRIMOVELENDERECO" => $parametros["NRIMOVELENDERECO"],
            "IDATIVO"          => $parametros["IDATIVO"],
            "CDOPERULTATU"     => $cdoperador,
            "NRORGULTATU"      => $nrorg,
            "NRENDERECOPARC"   => $nrenderecoparc,
            "CDPROSPECT"      => $cdprospect,
            "IDTPENDEPROS_OLD" => $tipoEnderecoOld
        );

        $this->businessPartnerFactory->updateENDEPROSPECT($params);
    }

    public function getCodigoClienteParceiro($nrparcnegocio)
    {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $dataset = $this->businessPartnerFactory->getCodigoClienteParceiro($params);
        return (count($dataset) > 0) ? $dataset[0]["CDCLIENTE"] : null;
    }

    public function getCodigoFornecedorParceiro($nrparcnegocio)
    {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $dataset = $this->businessPartnerFactory->getCodigoFornecedorParceiro($params);
        return (count($dataset) > 0) ? $dataset[0]["CDFORNECED"] : null;
    }

    public function getCodigoVendedorParceiro($nrparcnegocio)
    {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $dataset = $this->businessPartnerFactory->getCodigoVendedorParceiro($params);
        return (count($dataset) > 0) ? $dataset[0]["CDVENDEDOR"] : null;
    }

    public function getCodigoProspectParceiro($nrparcnegocio)
    {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $dataset = $this->businessPartnerFactory->getCodigoProspectParceiro($params);
        return (count($dataset) > 0) ? $dataset[0]["CDPROSPECT"] : null;
    }

    public function insertPARCERIA($nrparcnegocio, $cdtipoparceiro, $nrorg, $cdoperador)
    {
        $NRPARCERIA = $this->newCode->getCodeByTableName("PARCERIA", $nrorg, 1, 10, true);

        $dataTypes = array(
            "NRPARCERIA"     => $NRPARCERIA,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "CDTIPOPARCEIRO" => $cdtipoparceiro,
            "NRORG"          => $nrorg,
            "NRORGINCLUSAO"  => $nrorg,
            "IDATIVO"        => 'S',
            "CDOPERINCLUSAO" => $cdoperador,
            "NRPARCORIGEM"   => $nrparcnegocio
        );
        $this->businessPartnerFactory->insertPartnership($dataTypes);
    }

    public function updateParceria($idativo, $nrparcnegocio, $cdtipoparceiro, $nrorg, $cdoperador)
    {
        $params = array(
            "IDATIVO"        => $idativo,
            "NRORGULTATU"    => $nrorg,
            "CDOPERULTATU"   => $cdoperador,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "CDTIPOPARCEIRO" => $cdtipoparceiro
        );
        $this->businessPartnerFactory->updatePARCERIA($params);
    }

    public function updateParceriaNaoSelecionados(array $cdtipoparceiro, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $cdtipoparceiroString = implode("','", $cdtipoparceiro);
        $cdtipoparceiroString = "'$cdtipoparceiroString'";

        $params = array(
            "IDATIVO"       => "N",
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio
        );
        $this->businessPartnerFactory->updateParceriaNaoSelecionados($params, $cdtipoparceiroString);
    }

    public function insertCLIENTE($parametros, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $dataNascimentoFundacao = ($parametros['DTNASCIFUNDPARC'] == '') ? null : $parametros['DTNASCIFUNDPARC'];
        $arrayCLIENTE = array(
            "CDCLIENTE"     => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']] . $parametros['NRINSCRICAOPARC'],
            "IDTPIJURCLIE"  => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURCLIE"  => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCCLIE"  => $parametros['NMPRINCIPALPARC'],
            "NMFANTCLIE"    => $parametros['NMSECUNDARIPARC'],
            "NRORG"         => $nrorg,
            "NRORGINCLUSAO" => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "CDINSCESTA"    => $parametros['CDINSCESTA'],
            "CDINSCMUNI"    => $parametros['CDINSCMUNI'],
            "NRINSESTCLIE"  => $parametros['CDINSCESTA'],
            "NRINSMUNICLIE" => $parametros['CDINSCMUNI'],
            "IDTPLIMATEFAT" => "P",
            "IDATIVO"       => "S",
            "IDCLIEATIV"    => "S",
            "DTNASCCLIE"    => $dataNascimentoFundacao
        );
        $this->businessPartnerFactory->insertCLIENTE($arrayCLIENTE);
    }

    public function insertFORNECEDOR($parametros, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $arrayFORNECEDOR = array(
            "CDFORNECED"      => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']] . $parametros['NRINSCRICAOPARC'],
            "IDTPIJURFORN"    => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURFORN"    => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCFORN"    => $parametros['NMPRINCIPALPARC'],
            "NMFANTFORN"      => $parametros['NMSECUNDARIPARC'],
            "NRORG"           => $nrorg,
            "NRORGINCLUSAO"   => $nrorg,
            "CDOPERINCLUSAO"  => $cdoperador,
            "NRPARCNEGOCIO"   => $nrparcnegocio,
            "IDLIGEXTFORN"    => "N",
            "IDBLOQCMPSAFFOR" => "N",
            "IDATIVO"         => $parametros["IDATIVO"],
            "IDSITUCADA"      => ($parametros["IDATIVO"] == "S") ? "A" : "I"
        );
        $this->businessPartnerFactory->insertFORNECEDOR($arrayFORNECEDOR);
    }

    public function insertVENDEDOR($parametros, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $CDVENDEDOR = $this->newCode->getCodeByTableName("VENDEDOR", $nrorg, 1, 10, true);

        $arrayVENDEDOR = array(
            "CDVENDEDOR"     => $CDVENDEDOR,
            "IDTPIJURVEN"    => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURVEN"    => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCVEN"    => $parametros['NMPRINCIPALPARC'],
            "NMFANVEN"       => $parametros['NMSECUNDARIPARC'],
            "NRORG"          => $nrorg,
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador,
            "NRPARCNEGOCIO"  => $nrparcnegocio
        );

        $this->businessPartnerFactory->insertVENDEDOR($arrayVENDEDOR);
    }

    public function insertPROSPECT($parametros, $nrparcnegocio, $nrorg, $cdoperador)
    {
        $CDPROSPECT = $this->newCode->getCodeByTableName("PROSPECT", $nrorg, 1, 10, true);

        $arrayPROSPECT = array(
            "CDPROSPECT"     => $CDPROSPECT,
            "IDTPIJURPROS"   => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURPROS"   => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCPROS"   => $parametros['NMPRINCIPALPARC'],
            "NMFANTPROS"     => $parametros['NMSECUNDARIPARC'],
            "NRORG"          => $nrorg,
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador,
            "NRPARCORIGEM"   => $nrparcnegocio,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "IDATIVO"        => $parametros['IDATIVO']
        );

        $this->businessPartnerFactory->insertPROSPECT($arrayPROSPECT);
    }

    public function updateSpecificBusinessPartner($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $nrparcnegocio = $parametros["NRPARCNEGOCIO"];

        if ($parametros['editouInscricao']) {
            $this->validarInscricaoDuplicada($parametros['NRINSCRICAOPARC'], $parametros['CDTIPOPARCEIRO'], $nrparcnegocio);
        }

        $this->connection->beginTransaction();
        try {
            $dataParam = array(
                "NRPARCNEGOCIO"       => $nrparcnegocio,
                "CDSEGMER"            => $parametros['CDSEGMER'],
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
                "NMPRINCIPALPARC"     => $parametros['NMPRINCIPALPARC'],
                "NMSECUNDARIPARC"     => $parametros['NMSECUNDARIPARC'],
                "IDATIVO"             => $parametros['IDATIVO'],
                "DTNASCIFUNDPARC"     => ($parametros['DTNASCIFUNDPARC'] == '') ? null : $parametros['DTNASCIFUNDPARC'],
                "NRINSCRICAOPARC"     => $parametros['NRINSCRICAOPARC'],
                "CDTIPOPARCPRINCIPAL" => $parametros['CDTIPOPARCPRINCIPAL'],
                "CDTIPOINSCRICAO"     => $parametros['CDTIPOINSCRICAO'],
                "CDATIVCLI"           => $parametros['CDATIVCLI'],
                "IDPESSOAFISICA"      => $parametros['IDPESSOAFISICA'],
                "IDINSTITUICAO"       => $parametros['IDINSTITUICAO'],
                "NRSEQTPATIVCLI"      => $parametros['NRSEQTPATIVCLI'],
                "CDOPERRESPONSAVEL"   => $parametros['CDOPERRESPONSAVEL'],
                "NRTPCLASSIFICAPARC"  => $parametros['NRTPCLASSIFICAPARC'],
                "CDINSCESTA"          => $parametros['CDINSCESTA'],
                "CDINSCMUNI"          => $parametros['CDINSCMUNI'],
                "CDIDIOM"             => $parametros['CDIDIOM'],
                "NRMOEDA"             => $parametros['NRMOEDA']
            );

            $this->businessPartnerFactory->updateSpecificBusinessPartner($dataParam);

            $this->replicaParceiro($parametros, $nrparcnegocio, $nrorg, $cdoperador);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateCLIENTE($parametros, $nrorg, $cdoperador)
    {
        $dataNascimentoFundacao = ($parametros['DTNASCIFUNDPARC'] == '') ? null : $parametros['DTNASCIFUNDPARC'];
        $data = array(
            "NMRAZSOCCLIE"  => $parametros["NMPRINCIPALPARC"],
            "NMFANTCLIE"    => $parametros["NMSECUNDARIPARC"],
            "IDATIVO"       => $parametros["IDATIVO"],
            "IDCLIEATIV"    => $parametros["IDATIVO"],
            "CDINSCESTA"    => $parametros["CDINSCESTA"],
            "CDINSCMUNI"    => $parametros["CDINSCMUNI"],
            "NRINSESTCLIE"  => $parametros["CDINSCESTA"],
            "NRINSMUNICLIE" => $parametros["CDINSCMUNI"],
            "DTNASCCLIE"    => $dataNascimentoFundacao,
            "IDTPIJURCLIE"  => $this->typeOfPartner[$parametros["CDTIPOINSCRICAO"]],
            "NRINSJURCLIE"  => $parametros["NRINSCRICAOPARC"],
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"]
        );

        $this->businessPartnerFactory->updateCLIENTE($data);
    }

    public function inativarCliente($nrparcnegocio, $nrorg, $cdoperador)
    {
        $data = array(
            "IDATIVO"       => "N",
            "IDCLIEATIV"    => "N",
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $this->businessPartnerFactory->inativarCliente($data);
    }

    public function updateFORNECEDOR($parametros, $nrorg, $cdoperador)
    {
        $data = array(
            "NMRAZSOCFORN"  => $parametros["NMPRINCIPALPARC"],
            "NMFANTFORN"    => $parametros["NMSECUNDARIPARC"],
            "IDATIVO"       => $parametros["IDATIVO"],
            "IDSITUCADA"    => ($parametros["IDATIVO"] == "S") ? "A" : "I",
            "IDTPIJURFORN"  => $this->typeOfPartner[$parametros["CDTIPOINSCRICAO"]],
            "NRINSJURFORN"  => $parametros["NRINSCRICAOPARC"],
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"],
        );

        $this->businessPartnerFactory->updateFORNECEDOR($data);
    }

    public function inativarFornecedor($nrparcnegocio, $nrorg, $cdoperador)
    {
        $data = array(
            "IDATIVO"       => "N",
            "IDSITUCADA"    => "I",
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $this->businessPartnerFactory->inativarFornecedor($data);
    }

    public function updateVENDEDOR($parametros, $nrorg, $cdoperador)
    {
        $data = array(
            "NMRAZSOCVEN"   => $parametros["NMPRINCIPALPARC"],
            "NMFANVEN"      => $parametros["NMSECUNDARIPARC"],
            "IDATIVO"       => $parametros["IDATIVO"],
            "IDTPIJURVEN"   => $this->typeOfPartner[$parametros["CDTIPOINSCRICAO"]],
            "NRINSJURVEN"   => $parametros["NRINSCRICAOPARC"],
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"],
        );

        $this->businessPartnerFactory->updateVENDEDOR($data);
    }

    public function inativarVendedor($nrparcnegocio, $nrorg, $cdoperador)
    {
        $data = array(
            "IDATIVO"       => "N",
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $this->businessPartnerFactory->inativarVendedor($data);
    }

    public function updatePROSPECT($parametros, $nrorg, $cdoperador)
    {
        $data = array(
            "NMRAZSOCPROS"  => $parametros["NMPRINCIPALPARC"],
            "NMFANTPROS"    => $parametros["NMSECUNDARIPARC"],
            "IDATIVO"       => $parametros["IDATIVO"],
            "IDTPIJURPROS"  => $this->typeOfPartner[$parametros["CDTIPOINSCRICAO"]],
            "NRINSJURPROS"  => $parametros["NRINSCRICAOPARC"],
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"],
        );

        $this->businessPartnerFactory->updatePROSPECT($data);
    }

    public function inativarProspect($nrparcnegocio, $nrorg, $cdoperador)
    {
        $data = array(
            "IDATIVO"       => "N",
            "NRORGULTATU"   => $nrorg,
            "CDOPERULTATU"  => $cdoperador,
            "NRPARCNEGOCIO" => $nrparcnegocio
        );

        $this->businessPartnerFactory->inativarProspect($data);
    }

    //  ------------- FUNÇÕES  DE ENDEREÇO ------------------------
    public function getCountry()
    {
        return $this->businessPartnerFactory->getCountry();
    }

    public function getState($parametro)
    {
        return $this->businessPartnerFactory->getState($parametro);
    }

    public function getCity($parametro)
    {
        return $this->businessPartnerFactory->getCity($parametro);
    }

    public function getDistrict($parametro)
    {
        return $this->businessPartnerFactory->getDistrict($parametro);
    }

    public function findCodeDistrict($parametros)
    {
        return $this->businessPartnerFactory->findCodeDistrict($parametros);
    }

    public function getBusinessPartnerAddress($parametro)
    {
        return $this->businessPartnerFactory->getBusinessPartnerAddress($parametro);
    }

    public function getSpecificAddress($parametro)
    {
        return $this->businessPartnerFactory->getSpecificAddress($parametro);
    }

    public function insertNewAddress($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $this->connection->beginTransaction();

        try {
            $nrenderecoparc = $this->newCode->getCodeByTableName("ENDERECOPARC", $nrorg, 1, 10, true);

            $dataParam = array(
                "NRENDERECOPARC"   => $nrenderecoparc,
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDTIPOENDERECO"   => $parametros['CDTIPOENDERECO'],
                "NRORG"            => $nrorg,
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador,
                "CDPAIS"           => $parametros['CDPAIS'],
                "SGESTADO"         => $parametros['SGESTADO'],
                "CDMUNICIPIO"      => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"         => $parametros['CDBAIRRO'],
                "NMBAIRROENDERECO" => $parametros['NMBAIRROENDERECO'],
                "DSCOMPLEENDERECO" => $parametros['DSCOMPLEENDERECO'],
                "NRCEPENDERECO"    => $parametros['NRCEPENDERECO'],
                "DSENDERECO"       => $parametros['DSENDERECO'],
                "NRIMOVELENDERECO" => $parametros['NRIMOVELENDERECO'],
                "DSREFERENCIAENDE" => $parametros['DSREFERENCIAENDE']
            );
            $this->businessPartnerFactory->insertNewAddress($dataParam);

            $this->replicaEndereco($parametros, $nrenderecoparc, $parametros['NRPARCNEGOCIO'], $nrorg, $cdoperador);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteAddress(array $codigosEndereco)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($codigosEndereco as $nrenderecoparc) {
                $params = array(
                    "NRENDERECOPARC" => $nrenderecoparc
                );

                $this->businessPartnerFactory->deleteENDECLIE($params);
                $this->businessPartnerFactory->deleteENDEFORN($params);
                $this->businessPartnerFactory->deleteENDEVENDE($params);
                $this->businessPartnerFactory->deleteENDEPROSPECT($params);

                $this->businessPartnerFactory->deleteAddress($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateAddress($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getNrOrg();

        $this->connection->beginTransaction();
        try {
            $nrenderecoparc = $parametros['NRENDERECOPARC'];
            $nrparcnegocio = $parametros['NRPARCNEGOCIO'];

            if (!isset($parametros['IDATIVO'])) {
                $parametros['IDATIVO'] = 'S';
            }

            $dataParam = array(
                "NRENDERECOPARC"   => $nrenderecoparc,
                "NRPARCNEGOCIO"    => $nrparcnegocio,
                "CDTIPOENDERECO"   => $parametros['CDTIPOENDERECO'],
                "NRORGULTATU"      => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "CDPAIS"           => $parametros['CDPAIS'],
                "SGESTADO"         => $parametros['SGESTADO'],
                "CDMUNICIPIO"      => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"         => $parametros['CDBAIRRO'],
                "NMBAIRROENDERECO" => $parametros['NMBAIRROENDERECO'],
                "DSCOMPLEENDERECO" => $parametros['DSCOMPLEENDERECO'],
                "NRCEPENDERECO"    => $parametros['NRCEPENDERECO'],
                "DSENDERECO"       => $parametros['DSENDERECO'],
                "NRIMOVELENDERECO" => $parametros['NRIMOVELENDERECO'],
                "DSREFERENCIAENDE" => $parametros['DSREFERENCIAENDE']
            );
            $this->businessPartnerFactory->updateAddress($dataParam);

            $this->replicaEndereco($parametros, $nrenderecoparc, $nrparcnegocio, $nrorg, $cdoperador);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE comunicação ------------------------
    public function getWayOfCommunication($parametro)
    {
        return $this->businessPartnerFactory->getWayOfCommunication($parametro);
    }

    public function getListWaysOfCommunication()
    {
        return $this->businessPartnerFactory->getListWaysOfCommunication();
    }

    public function getSpecificWayOfCommunication($parametro)
    {
        return $this->businessPartnerFactory->getSpecificWayOfCommunication($parametro);
    }

    public function insertNewCommunicationData($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            $NRCOMUNICAPARC = $this->newCode->getCodeByTableName("COMUNICAPARC", $nrorg, 1, 10, true);

            $dataParam = array(
                "NRCOMUNICAPARC"   => $NRCOMUNICAPARC,
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDFORMACOMU"      => $parametros['CDFORMACOMU'],
                "NRENDERECOPARC"   => null,
                "CDPREFIXCOMUPARC" => $parametros['CDPREFIXCOMUPARC'],
                "CDCOMUNICAPARC"   => $parametros['CDCOMUNICAPARC'],
                "NRORG"            => $nrorg,
                "DTULTATU"         => null,
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador,
                "NRORGULTATU"      => null,
                "CDOPERULTATU"     => null,
                "CDCOMPLECOMUPARC" => null,
                "DSCONTATO"        => null
            );
            $this->businessPartnerFactory->insertNewCommunicationData($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteWayOfCommunication($parametro)
    {
        $this->connection->beginTransaction();
        try {
            $this->businessPartnerFactory->deleteWayOfCommunication($parametro);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateWayOfCommunication($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $dataParam = array(
                "NRCOMUNICAPARC"   => $parametros['NRCOMUNICAPARC'],
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDFORMACOMU"      => $parametros['CDFORMACOMU'],
                "NRENDERECOPARC"   => null,
                "CDPREFIXCOMUPARC" => $parametros['CDPREFIXCOMUPARC'],
                "CDCOMUNICAPARC"   => $parametros['CDCOMUNICAPARC'],
                "NRORGULTATU"      => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "CDCOMPLECOMUPARC" => null,
                "DSCONTATO"        => null
            );
            $this->businessPartnerFactory->updateWayOfCommunication($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE INTERLOCUTOR ------------------------

    public function getSectors()
    {
        return $this->businessPartnerFactory->getSectors();
    }

    public function getPositionsOfInterlocutors()
    {
        return $this->businessPartnerFactory->getPositionsOfInterlocutors();
    }

    public function getInterlocutors($parametro)
    {
        return $this->businessPartnerFactory->getInterlocutors($parametro);
    }

    public function getSpecificInterlocutor($parametro)
    {
        return $this->businessPartnerFactory->getSpecificInterlocutor($parametro);
    }

    public function insertNewInterlocutor($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            $NRSEQINTERLOCUTOR = $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", $nrorg, 1, 10, true);

            $dataParam = array(
                "NRSEQINTERLOCUTOR" => $NRSEQINTERLOCUTOR,
                "NMINTERLOCUTOR"    => $parametros['NMINTERLOCUTOR'],
                "NRPARCNEGOCIO"     => $parametros['NRPARCNEGOCIO'],
                "EMAILINTERLOC"     => $parametros['EMAILINTERLOC'],
                "NRTELEFONE1"       => $parametros['NRTELEFONE1'],
                "NRTELEFONE2"       => $parametros['NRTELEFONE2'],
                "NRTELEFONE3"       => $parametros['NRTELEFONE3'],
                "NRTELEFONE4"       => $parametros['NRTELEFONE4'],
                "CDCARGOINTERLOC"   => $parametros['CDCARGOINTERLOC'],
                "DTNASCIMENTO"      => $parametros['DTNASCIMENTO'],
                "IDRECEBEPRESENTE"  => $parametros['IDRECEBEPRESENTE'],
                "CDPAIS"            => $parametros['CDPAIS'],
                "SGESTADO"          => $parametros['SGESTADO'],
                "CDMUNICIPIO"       => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"          => $parametros['CDBAIRRO'],
                "NRCEPINTERLOCUTOR" => $parametros['NRCEPINTERLOCUTOR'],
                "DSENDEINTERLOCUTOR" => $parametros['DSENDEINTERLOCUTOR'],
                "DSCOMPINTERLOCUTOR" => $parametros['DSCOMPINTERLOCUTOR'],
                "NRENDEINTERLOCUTOR" => $parametros['NRENDEINTERLOCUTOR'],
                "NRORG"             => $nrorg,
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador,
                "CDSETOR"           => $parametros['CDSETOR'],
                "NRCPFINTERLOC"     => $parametros['NRCPFINTERLOC'],
                "IDTPSIGNATARIO"    => $parametros['IDTPSIGNATARIO'],
                "IDNOMEVINDODAAPI"  => "N"
            );
            $this->businessPartnerFactory->insertNewInterlocutor($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateInterlocutor($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            $dataParam = array(
                "NRSEQINTERLOCUTOR" => $parametros['NRSEQINTERLOCUTOR'],
                "NMINTERLOCUTOR"    => $parametros['NMINTERLOCUTOR'],
                "NRPARCNEGOCIO"     => $parametros['NRPARCNEGOCIO'],
                "EMAILINTERLOC"     => $parametros['EMAILINTERLOC'],
                "NRTELEFONE1"       => $parametros['NRTELEFONE1'],
                "NRTELEFONE2"       => $parametros['NRTELEFONE2'],
                "NRTELEFONE3"       => $parametros['NRTELEFONE3'],
                "NRTELEFONE4"       => $parametros['NRTELEFONE4'],
                "CDCARGOINTERLOC"   => $parametros['CDCARGOINTERLOC'],
                "DTNASCIMENTO"      => $parametros['DTNASCIMENTO'],
                "IDRECEBEPRESENTE"  => $parametros['IDRECEBEPRESENTE'],
                "CDPAIS"            => $parametros['CDPAIS'],
                "SGESTADO"          => $parametros['SGESTADO'],
                "CDMUNICIPIO"       => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"          => $parametros['CDBAIRRO'],
                "NRCEPINTERLOCUTOR" => $parametros['NRCEPINTERLOCUTOR'],
                "DSENDEINTERLOCUTOR" => $parametros['DSENDEINTERLOCUTOR'],
                "DSCOMPINTERLOCUTOR" => $parametros['DSCOMPINTERLOCUTOR'],
                "NRENDEINTERLOCUTOR" => $parametros['NRENDEINTERLOCUTOR'],
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdoperador,
                "CDSETOR"           => $parametros['CDSETOR'],
                "NRCPFINTERLOC"     => $parametros['NRCPFINTERLOC'],
                "IDTPSIGNATARIO"    => $parametros['IDTPSIGNATARIO'],
                "IDNOMEVINDODAAPI"  => "N"
            );
            $this->businessPartnerFactory->updateInterlocutor($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteInterlocutor($parametro)
    {
        $this->connection->beginTransaction();
        try {
            $this->businessPartnerFactory->deleteInterlocutor($parametro);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE DADOS BANCÁRIOS  ------------------------
    public function getBankDetails($parametro)
    {
        return $this->businessPartnerFactory->getBankDetails($parametro);
    }

    public function getBanks()
    {
        return $this->businessPartnerFactory->getBanks();
    }

    public function getAgencies($parametro)
    {
        return $this->businessPartnerFactory->getAgencies($parametro);
    }

    public function getCheckingAccount($parametro)
    {
        return $this->businessPartnerFactory->getCheckingAccount($parametro);
    }

    public function getTypeOfCheckingAccount()
    {
        return $this->businessPartnerFactory->getTypeOfCheckingAccount();
    }

    public function getSpecificBankDetails($parametro)
    {
        return $this->businessPartnerFactory->getSpecificBankDetails($parametro);
    }

    public function getBranches()
    {
        return $this->businessPartnerFactory->getBranches();
    }

    public function getInvoiceSeries()
    {
        return $this->businessPartnerFactory->getInvoiceSeries();
    }

    public function insertNewBankDetails($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();
        try {
            $NRCONTCORRPARC = $this->newCode->getCodeByTableName("CONTCORRPARC", $nrorg, 1, 10, true);
            $dataParam = array(
                "NRCONTCORRPARC" => $NRCONTCORRPARC,
                "NRORG"          => $nrorg,
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                "CDBANCO"        => $parametros['CDBANCO'],
                "CDAGENCIA"      => $parametros['CDAGENCIA'],
                "CDCONTCORR"     => $parametros['CDCONTCORR'],
                "NRTPCONTCORR"   => $parametros['NRTPCONTCORR'],
                "NRORGINCLUSAO"  => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador
            );
            $this->businessPartnerFactory->insertNewBankDetails($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteBankDetails($parametro)
    {
        $this->connection->beginTransaction();
        try {
            $this->businessPartnerFactory->deleteBankDetails($parametro);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateBankDetails($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();

        try {
            //$NRCONTCORRPARC = $this->newCode->getCodeByTableName("CONTCORRPARC", $nrorg, 1, 10, true);
            $dataParam = array(
                "NRCONTCORRPARC" => $parametros['NRCONTCORRPARC'],
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                "CDBANCO"        => $parametros['CDBANCO'],
                "CDAGENCIA"      => $parametros['CDAGENCIA'],
                "CDCONTCORR"     => $parametros['CDCONTCORR'],
                "NRTPCONTCORR"   => $parametros['NRTPCONTCORR'],
                "NRORGULTATU"  => $nrorg,
                "CDOPERULTATU" => $cdoperador
            );
            $this->businessPartnerFactory->updateBankDetails($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE PARÂMETROS DE FATURAMENTO  ------------------------

    public function getBillingParameter($parametro)
    {
        return $this->businessPartnerFactory->getBillingParameter($parametro);
    }

    public function billingParameter($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();
        try {
            if ($parametros['NRPARAMFATURA'] == null) {
                $NRPARAMFATURA = $this->newCode->getCodeByTableName("PAR_PARAMFATURA", $nrorg, 1, 10, true);
                $dataParam = array(
                    "NRPARAMFATURA"  => $NRPARAMFATURA,
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "CDFILIALFATURA" => $parametros['CDFILIALFATURA'],
                    "CDSERIENF"      => $parametros['CDSERIENF'],
                    "CDEMPRESA"      => $parametros['CDEMPRESA'],
                    "CDINSCESTA"     => $parametros['CDINSCESTA'],
                    "CDINSCMUNI"     => $parametros['CDINSCMUNI'],
                    "NRORG"          => $nrorg,
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                $this->businessPartnerFactory->insertBillingParameter($dataParam);
            } else {
                $NRPARAMFATURA = $parametros['NRPARAMFATURA'];
                $dataParamUpdate = array(
                    "NRPARAMFATURA"  => $parametros['NRPARAMFATURA'],
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "CDFILIALFATURA" => $parametros['CDFILIALFATURA'],
                    "CDSERIENF"      => $parametros['CDSERIENF'],
                    "CDEMPRESA"      => $parametros['CDEMPRESA'],
                    "CDINSCESTA"     => $parametros['CDINSCESTA'],
                    "CDINSCMUNI"     => $parametros['CDINSCMUNI'],
                    "NRORGULTATU"    => $nrorg,
                    "CDOPERULTATU"   => $cdoperador
                );
                $this->businessPartnerFactory->updateBillingParameter($dataParamUpdate);
            }
            $this->connection->commit();
            return $NRPARAMFATURA;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE logotipo   ------------------------
    public function sendImage($parametros)
    {
        $this->connection->beginTransaction();
        try {
            //Remove a imagem antiga
            $dataParam = array(
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO']
            );
            $caminhoAntigo = $this->getImagesLogotype($dataParam);
            if (!empty($caminhoAntigo[0]["DSDIRLOGOMARCA"])) {
                $this->FileServerService->deleteFile($caminhoAntigo[0]["DSDIRLOGOMARCA"]);
            }
            //Insere a imagem Nova
            $link = '';
            $row = $this->FileServerService->uploadFiles($parametros['DSDIRLOGOMARCA'], 'anexo_parceiro');
            foreach ($row as $file) {
                $dataParam = array(
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "DSDIRLOGOMARCA" => $file['path'],
                    "DSTYPELOGOTIPO" => $file['type'],
                    "QTSIZELOGOTIPO" => $file['size']
                );
                $this->businessPartnerFactory->sendImage($dataParam);
                $link = $file['path'];
            }
            $this->connection->commit();
            return $link;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteImage($parametros)
    {
        $caminhoAntigo = $parametros['LINK'];
        $this->FileServerService->deleteFile($caminhoAntigo);
        $dataParam = array(
            "NRPARCNEGOCIO" => $parametros['NRPARCNEGOCIO']
        );
        $this->businessPartnerFactory->deleteImage($dataParam);
    }

    public function getImagesLogotype($parametro)
    {
        return $this->businessPartnerFactory->getImagesLogotype($parametro);
    }

    public function getBusinessPartnerStandardFinancialIndices($params)
    {
        return $this->businessPartnerFactory->getBusinessPartnerStandardFinancialIndices($params);
    }

    public function salvarIndiceFinanceiroPadraoParceiro($nrindicespadrparc, $nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        if (empty($nrindicespadrparc)) {
            $this->inserirIndiceFinanceiroPadraoParceiro($nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $nrorg, $cdoperador);
        } else {
            $this->atualizarIndiceFinanceiroPadraoParceiro($cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $cdoperador, $nrorg, $nrindicespadrparc);
        }
    }

    private function inserirIndiceFinanceiroPadraoParceiro($nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $nrorg, $cdoperador)
    {
        $nrindicespadrparc = $this->newCode->getCodeByTableName("FCT_INDICESPADRPARC", $nrorg, 1, 10, true);
        $paramsInsert = array(
            "NRINDICESPADRPARC" => $nrindicespadrparc,
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "CDTPINDICE"        => $cdtpindice,
            "IDPERVALOR"        => $idpervalor,
            "CDINDICE"          => $cdindice,
            "NRPERCENTINDICE"   => $nrpercentindice,
            "NRORG"             => $nrorg,
            "IDATIVO"           => "S",
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperador,
        );
        $this->businessPartnerFactory->inserirIndiceFinanceiroPadraoParceiro($paramsInsert);
    }

    private function atualizarIndiceFinanceiroPadraoParceiro($cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $cdoperador, $nrorg, $nrindicespadrparc)
    {
        $paramsUpdate = array(
            "CDTPINDICE"        => $cdtpindice,
            "IDPERVALOR"        => $idpervalor,
            "CDINDICE"          => $cdindice,
            "NRPERCENTINDICE"   => $nrpercentindice,
            "CDOPERULTATU"      => $cdoperador,
            "NRORGULTATU"       => $nrorg,
            "NRINDICESPADRPARC" => $nrindicespadrparc,
            "NRORG"             => $nrorg
        );
        $this->businessPartnerFactory->atualizarIndiceFinanceiroPadraoParceiro($paramsUpdate);
    }

    public function excluirIndiceFinanceiroPadraoParceiro(array $nrindicespadrparc)
    {
        $nrorg = $this->environment->getNrOrg();
        $nrindicespadrparc = implode(",", $nrindicespadrparc);

        $this->businessPartnerFactory->excluirIndiceFinanceiroPadraoParceiro($nrindicespadrparc, $nrorg);
    }

    public function getSomaPercentualIndicesParceiro($nrparcnegocio, $nrindicespadrparc)
    {
        $params = array(
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "NRINDICESPADRPARC" => $nrindicespadrparc
        );
        $dataset = $this->businessPartnerFactory->getSomaPercentualIndicesParceiro($params);
        return (float)$dataset[0]["NRPERCENTINDICE"];
    }

    public function validarIndiceDuplicadoParceiro($nrparcnegocio, $nrindicespadrparc, $cdindice, $cdtpindice, $idpervalor)
    {
        $params = array(
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "CDTPINDICE"        => $cdtpindice,
            "IDPERVALOR"        => $idpervalor,
            "CDINDICE"          => $cdindice,
            "NRINDICESPADRPARC" => $nrindicespadrparc
        );
        $dataset = $this->businessPartnerFactory->validarIndiceDuplicadoParceiro($params);
        if ((int)$dataset[0]["EXISTE"] > 0) {
            throw new \Exception("O Índice Financeiro informado já está cadastrado para este Parceiro de Negócio.");
        }
    }

    public function inserirBairro($cdpais, $sgestado, $cdmunicipio, $nmbairro)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $newCodeKey = "BAIRRO" . $cdpais . $sgestado . $cdmunicipio;

        $cdbairro = $this->newCode->getCodeByTableName($newCodeKey, $nrorg);
        $params = array(
            "CDPAIS"         => $cdpais,
            "SGESTADO"       => $sgestado,
            "CDMUNICIPIO"    => $cdmunicipio,
            "CDBAIRRO"       => $cdbairro,
            "NMBAIRRO"       => $nmbairro,
            "NRORG"          => $nrorg,
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador
        );
        $this->businessPartnerFactory->inserirBairro($params);

        return $cdbairro;
    }

}
